use crate::Vec2;

//odd-q vertical layout https://www.redblobgames.com/grids/hexagons
pub struct HexMap<T: Default> {
    size: Vec2<i32>,
    data: Vec<T>,
    tile_size: Vec2<i32>,
}

impl<T: Default + Clone> HexMap<T> {
    //pub static
    pub fn new(size: Vec2<i32>, tile_size: Vec2<i32>) -> Self {
        HexMap {
            size,
            data: vec![T::default(); size.size()],
            tile_size,
        }
    }

    //pub
    pub fn get(&self, coord: Vec2<i32>) -> &T {
        &self.data[self.coord_to_idx(coord)]
    }

    pub fn get_mut(&mut self, coord: Vec2<i32>) -> &mut T {
        let idx = self.coord_to_idx(coord);
        &mut self.data[idx]
    }

    pub fn set(&mut self, coord: Vec2<i32>, val: T) {
        let idx = self.coord_to_idx(coord);
        self.data[idx] = val;
    }

    pub fn size(&self) -> Vec2<i32> {
        self.size
    }

    //priv
    fn coord_to_idx(&self, coord: Vec2<i32>) -> usize {
        (coord.x + coord.y * self.size.x) as usize
    }
}
